/********************************************************************************/
/*																				*/
/* DSP_ex1.c                                	                            	*/
/*																				*/
/* Example1 for a HERON processor module used in the "getting started" example 	*/
/* for the HERON-FPGA and HERON-IO modules. With the FPGA configured with		*/
/* the correct bitstream - this test writes data and checks it can read it		*/
/* back.																		*/
/*																				*/
/*	   2001 Hunt Engineering													*/
/*	    All Rights Reserved														*/
/*																				*/
/* 	Ver	Date		Author	Comments											*/
/*	---	----		------	--------											*/
/*																				*/
/*	1.0	14/12/00	RASW	Created.											*/
/*  1.1 12-2-01     PW	    tidied and formatted								*/ 
/********************************************************************************/


#include <stdio.h>
#include <std.h>
#include <swi.h>
#include <tsk.h>
#include "heron4.h"

#pragma FUNC_EXT_CALLED(maintask)


/************************************/
/* FIFO information for Heron_API   */
/************************************/
  
#define BOARD  0x0  /* ID of board being used */
HeronFIFO *rfifo;		/* Handle for read FIFO  */
HeronFIFO *wfifo;   		/* Handle for write FIFO */
HeronHSB *hsb;          /* handle for HSB accesses */       
#define USER_WRITE			8
#define USER_READ			9      
#define USER_READ_REPLY		10              



/************************************/
/* Arrays for data in FPGA test     */
/************************************/
#define ARRAY_SIZE 4096		/* number of words to write to FPGA*/
int *pattern;			/* source array */
int *results;			/* results read back from FPGA */
int slot;
int i;
       
int main(void)
{
}


/************************************/
/* Initialise Arrays for FPGA test 	*/
/* Source array is set to a ramp	*/
/* Destination is a constant		*/
/************************************/
void init_array(int pattern[ARRAY_SIZE], int results[ARRAY_SIZE])
{
	int n;
	printf("Initialising Arrays\n");
	for(n=0;n<ARRAY_SIZE;n++)
		{
			pattern[n]=n;
			results[n]=0xc0ffee;
		}
}



/************************************/
/* Verify results by comparing 		*/
/* source & destination arrays - if	*/
/* different, print on screen.		*/
/************************************/
void test_results(int pattern[ARRAY_SIZE], int results[ARRAY_SIZE])
{
	int n;
	printf("Verifying Results\n");
	for(n=0;n<ARRAY_SIZE;n++)
		{
			if(pattern[n]!=results[n])
			printf("Error: %d\tWrote %08x\tRead %08x\n", n, pattern[n],results[n]);
		}
}   

         
 
 
/********************************************************************************/
/*																				*/
/*	readback_user																*/
/*																				*/
/*	This function is used to read back from the FPGA							*/
/*																				*/
/********************************************************************************/
int readback_user(unsigned short slot, unsigned char *reply, int buffer_size, int user_addr)
{
	int message_type;
	unsigned char send_data[5];   
	unsigned int done;
	int status;

	send_data[0]=user_addr;
	send_data[1]=1;


   	status = HeronHsbSendMessage(hsb,USER_READ,BOARD,slot,send_data,2);  
   	status = HeronHsbReceiveMessage(hsb,&message_type,BOARD,slot,reply,buffer_size,&done);
	if (status == HERON_OK) 
	{
		if (message_type != USER_READ_REPLY) 
		{
			printf("Error: expected user read reply   \n");
			return(-1);
		}
		else 
		{       
			if (done!=1)
			{
				printf("Error: expected one byte, recieved %d  \n",done);
				return(-1);
			}
			else
			{	
			return(reply[0]);
			}
		}
	}

	return(status);
}    



/****************************************************/
/* makes an HSB message to send to the FPGA module	*/
/* it is a "user" type message to write to the      */
/* application FPGA. In the example1 bitstream address */
/* 0 sets the FIFO to use for input to the FPGA, and */
/* address 1 sets the FIFO to use for writes from the */
/* FPGA												*/
/************************************/


void SendUserData(unsigned short slot, int Address, int Data)
{
unsigned char user[2];
unsigned char rec_data[3];
int reply;
int status;

	/* Send user data */
	user[0]=Address;
	user[1]=Data;
        
	status = HeronHsbSendMessage(hsb,USER_WRITE,BOARD,slot,user,2);        
        
        
	if(status != HERON_OK) 
		{
		printf("HSB Write module failed with %x, exiting\n", status); 
		}
/* read it back to check*/        
    reply =readback_user(slot,rec_data,2,Address);
    if(reply!=Data)printf("error hsb readback failed with %x\n",reply); 

}         
                  

/************************************/
/* Maintask - main processing task	*/
/* Performs the test by writing the  test pattern 	*/
/* and reading it back.				*/
/************************************/

int maintask(void)
{
int status;              
char tmp[10];  
int fifo;
int dsp_fifo_no;          
          
printf("Select the FIFO that the DSP should use? (0..5) ");
gets(tmp);
dsp_fifo_no = atoi(tmp);
if( (dsp_fifo_no<0)||(dsp_fifo_no>5))
		{
	 	printf("Bad FIFO\n");
	 	exit(-1);
	 	}          

printf("What slot is the FPGA module fitted to? (1..4) ");
gets(tmp);
slot = atoi(tmp);
if( (slot<1)||(slot>4))
		{
	 	printf("Bad Slot number\n");
	 	exit(-1);
	 	}   

pattern = (int *) malloc(ARRAY_SIZE * sizeof(int));
results = (int *) malloc(ARRAY_SIZE * sizeof(int));
if ( (pattern==0) || (results==0) )
	{
	printf("Memory allocation error, exiting...\n");
	exit(0);
	}                          

/* open the HSB device to be able to set the FIFO number for the FPGA example1 */
hsb = HeronHsbOpen();
if (hsb == NULL) {
	printf("Error: Call to HeronHsbOpen failed, error = %d\n",heronerr); 
	exit(0);
}

/* open the FIFO for reading*/
    rfifo=HeronOpenFifo(dsp_fifo_no,"r");
    if (rfifo==NULL)
     {
     printf("Cannot open fifo %d for read. Error %d\n",dsp_fifo_no,heronerr);
     exit(-1); 	       
     }

/* open the FIFO for writing*/
    wfifo=HeronOpenFifo(dsp_fifo_no,"w");
    if (wfifo==NULL)
     {
     printf("Cannot open fifo %d for write. Error %d\n",dsp_fifo_no,heronerr);
     exit(-1); 	
     } 

/* Initialise arrays */
	init_array(pattern, results);

/* Start I/O System */
	HeronConfigOff();

     
/* Set up the FPGA for the demo */	
	// 0x40 = FIFO6
	// 0x20 = FIFO5
	// 0x10 = FIFO4
	// 0x08 = FIFO3
	// 0x04 = FIFO2
	// 0x02 = FIFO1
	// 0x01 = FIFO0
	printf("Select the input FIFO number that the FPGA should use? (0..5) ");
	gets(tmp);
	fifo = atoi(tmp);
	if( (fifo<0)||(fifo>5))
		if( (fifo<0)||(fifo>5))
		{
	 	printf("Bad FIFO\n");
	 	exit(-1);
	 	} 
	fifo=1<<fifo;
	SendUserData(slot, 0, fifo);
	printf("Selected Input FIFO with control word %02x\n",fifo);

	printf("Select the output FIFO number that the FPGA should use? (0..5) ");
	gets(tmp);
	fifo = atoi(tmp);
	if( (fifo<0)||(fifo>5))
		{
	 	printf("Bad FIFO\n");
	 	exit(-1);
	 	}
	fifo= 1<<fifo;
	SendUserData(slot, 1, fifo);
	printf("Selected Output FIFO with control word %02x\n",fifo);


	/* Ask user if he wants to proceed */
	printf("Hit a key to start testing FIFO data");
	getchar();
	printf("OK, Testing...\n");   
     
     

while(1)
{
	printf("Writing Test Pattern\n");
  	status = HeronWrite(wfifo,pattern,ARRAY_SIZE);  
  	if (status != HERON_IO_IN_PROGRESS)
  	{
  		printf("error cannot start the write\n");
  		exit(-1);
  	}
 
	printf("Writing has started -- now starting the read\n");  
  	status = HeronRead(rfifo,results,ARRAY_SIZE);
	if (status != HERON_IO_IN_PROGRESS)
	{      
		printf("error cannot start the read\n");
  		exit(-1);
  	}                  

	status = HeronWaitIo(wfifo); 
    if (status != HERON_OK) 
	{
		printf("Cannot write fifo %d. Error %d\n",dsp_fifo_no,heronerr);
			exit(-1);
	}                           
	printf("Written Test Pattern... reading back\n");
	status = HeronWaitIo(rfifo);
   	if (status != HERON_OK) 
	{
		printf("Cannot read fifo %d. Error %d\n",dsp_fifo_no,heronerr);
		exit(-1);
	} 
	printf("Reading Test Pattern complete\n");		
			
	
	test_results(pattern,results);
    printf("Test Complete\n");
}             
                          
/* in the example we will never reach here but this is what we should remember to do if we did */                          
status = HeronClose(rfifo);
status = HeronClose(wfifo);     
status = HeronHsbClose(hsb);        
             
             
return(0);
}
